from inspect import getframeinfo, getmodule
from sys import _getframe
import traceback
from ludibrio._testdouble import _TestDouble
from helpers import frameOutOfContext

class TraceRoute(_TestDouble):
    def __init__(self):
        self.__traceback__= []

    def remember(self):
        frame = self._frame_of_trace()
        traceinfo = getframeinfo(frame)
        if self._is_equal_to_last(traceinfo):
            self.__traceback__.append(traceinfo)

    def _is_equal_to_last(self, trace):
        return bool(not self.__traceback__
                    or not self.__traceback__[-1].filename == trace.filename
                    or not self.__traceback__[-1].lineno == trace.lineno)
            
    def _frame_of_trace(self):
        return frameOutOfContext()

    def stack_trace(self, limit=10):
        """Format a stack trace and return."""
        return 'Stack trace (most recent call last)\n''  '+''.join(
                    traceback.format_list(self._extracted_list()[:limit])
                    ).strip()

    def _extracted_list(self):
        """return a list of strings ready for printing."""
        return [[route.filename,
                 route.lineno,
                 route.function,
                 self._format_code_context(route.code_context)]
                     for route in self.__traceback__]
             

    def stack_code(self, limit=10):
        return '\n'.join(
                [self._format_code_context(route.code_context)
                    for route in self.__traceback__[:limit]])
    
    def _format_code_context(self, code_context):
        return '\n'.join([line.strip() for line in code_context]) 
    
    def most_recent_call(self):
        last_code = self.__traceback__[-1].code_context
        return self._format_code_context(last_code)    
    
    def __methodCalled__(self, *args, **kargs):
        self.remember()
        return self

    def __iter__(self):
        self.remember()
        yield self

    def __str__(self):
        self.remember()
        return self.__kargs__.get('str', 'TraceRoute Object')

    def __int__(self):
        self.remember()
        return self.__kargs__.get('int', 1)

    def __float__(self):
        self.remember()
        return self.__kargs__.get('float', 1.0)

    def __nonzero__(self):
        self.remember()
        return True

    def __getattr__(self, x):
        self.remember()
        return self




#-*- coding:utf-8 -*-

from inspect import getframeinfo
from sys import _getframe as getframe
from _testdouble import _TestDouble
from dependencyinjection import DependencyInjection


STOPRECORD = False
RECORDING = True


class Stub(_TestDouble):
    """Stubs provides canned answers to calls made during the test.
    """
    __expectation__= [] # [(attribute, args, kargs),]
    __recording__ = RECORDING
    __lastPropertyCalled__ = None
    __dependency_injection__ = None

    def __enter__(self):
        self.__expectation__= []
        self.__recording__ = RECORDING
        self.__dependency_injection__ = DependencyInjection(double = self)
        return self

    def __methodCalled__(self, *args, **kargs):
        property_name = self._property_called_name()
        return self._property_called(property_name, args, kargs)

    def _property_called_name(self):
        propertyCalledName =  self.__lastPropertyCalled__ or getframeinfo(getframe(2))[2]
        self.__lastPropertyCalled__ = None
        return propertyCalledName

    def _property_called(self, property, args=[], kargs={}, response=None):
        if self.__recording__:
            self._new_expectation([property, args, kargs, response])
            return self
        else:
            return self._expectation_value(property, args, kargs)

    def __exit__(self, type, value, traceback):
        self.__dependency_injection__.restoure_import()
        self.__recording__ = STOPRECORD

    def __setattr__(self, attr, value):
        if attr in dir(Stub):
            object.__setattr__(self, attr, value)
        else:
            self._property_called('__setattr__', args=[attr, value])

    def _new_expectation(self, attr):
        self.__expectation__.append(attr)

    def __rshift__(self, response):
            self.__expectation__[-1][3] = response
    __lshift__ = __rshift__

    def _expectation_value(self, attr, args=[], kargs={}):
        for position, (attrExpectation, argsExpectation, kargsExpectation, response) in enumerate(self.__expectation__):
            if (attrExpectation, argsExpectation, kargsExpectation) == (attr, args, kargs):
                self._to_the_end(position)
                return response
        if self.__kargs__.has_key('proxy'):
            return getattr(self.__kargs__.get('proxy'), attr)(*args, **kargs)
        raise AttributeError("Stub Object received unexpected call")

    def _to_the_end(self, position):
        self.__expectation__.append(self.__expectation__.pop(position))

    def __getattr__(self, x):
        self.__lastPropertyCalled__ = x
        return self._property_called('__getattribute__', (x,), response=self)
    
    def __del__(self):
        self.__dependency_injection__.restoure_object()
    
    def restoure_import(self):
        self.__dependency_injection__.restoure_object()



#!/usr/bin/env python
#-*- coding:utf-8 -*-

from sys import _getframe
import os


def frameOutOfContext():
    this_frame = frame = _getframe(1)
    while folder() in frame.f_code.co_filename:
        frame = frame.f_back
    return frame

def folder():
    return os.path.dirname(__file__)
    





from ludibrio.specialarguments import matcher
import re

class ParameterException(Exception):
    """'Exception' for mandatory parameters"""

@matcher
def any(x, y):
    return True

@matcher
def equal_to(x, y):
    if  x == y: return True
    raise ParameterException("%r is not equal to %r"%(x, y))

@matcher
def into(item, container):
    if item in container: return True
    raise ParameterException("%r is not in %r"%(item, container))

@matcher
def contains(container, item):
    if item in container: return True
    raise ParameterException("%r is not contains %r"%(container, item))

@matcher
def greater_than(x, y):
    if x > y: return True
    raise ParameterException("%r is not greater than %r"%(x, y))

@matcher
def greater_than_or_equal_to(x, y):
    if x >= y: return True
    raise ParameterException("%r is not greater than or equal to %r"%(x, y))

@matcher
def less_than(x, y):
    if x < y: return True
    raise ParameterException("%r is not less than %r"%(x, y))

@matcher
def less_than_or_equal_to(x, y):
    if x <= y: return True
    raise ParameterException("%r is not less than or equal to %r"%(x, y))

@matcher
def in_any_order(container, elements):
    for element in elements:
        if element not in container:
            raise ParameterException("%r does not have in any order %r"%(container, elements))
    return True

@matcher
def any_of(container, elements):
    for element in elements:
        if element in container:
            return True
    raise ParameterException("%r does not have any of %r"%(container, elements))

@matcher
def kind_of(obj, kind):
    if isinstance(obj, kind): return True
    raise ParameterException("%r is not a kind of %r"%(obj, kind))

@matcher
def instance_of(obj, kind):
    if isinstance(obj, kind): return True
    raise ParameterException("%r is not a instance of %r"%(obj, kind))

@matcher
def ended_with(x, y):
    if x.endswith(y): return True
    raise ParameterException("%r is not ended with %r"%(x, y))

@matcher
def started_with(x, y):
    if x.startswith(y): return True
    raise ParameterException("%r is not started with %r"%(x, y))


@matcher
def like(string, regex):
    if re.match(regex, string) is not None:
        return True
    raise ParameterException("%r is not like %r"%(string, regex))


@matcher
def equal_to_ignoring_case(x, y):
    if unicode(x, 'utf-8').lower() == unicode(y, 'utf-8').lower():
        return True
    raise ParameterException("%r is not equal to %r ignoring case"%(x, y))


